/*********************************************************************
 * This file is part of the Compod project
 * Contact: Alexander Nussbaumer (AN, alnuss)
 *   <alexander.nussbaumer@uni-graz.at>, <alnuss@gmx.net>
 *   
 *  modified: AN, 20070916
 *********************************************************************/

package isac.session;


/**
 * This is a dummy implementation of the ICompod interface.
 * In contrast to a "real" implemenatin, this class does not
 * connect the Compod services.
 * 
 * @author Alexander Nussbaumer (AN)
 */
public class CompodDummy implements ICompod
{
	// --- fields ---
	
	/**
	 * The lmsid. this is not needed in the dummy, however, the dummy
	 * simulates a 'real' compod interface, so it is neccessary to insist
	 * of it's usage (exception if not set).
	 */
	private String lmsid_ = null;

	/**
	 * isac objects. real object IDs, but only a few in the dummy.
	 */
	private String[] objects_ = 
	{ "exp_IsacCore_Tests_1a.xml", "exp_IsacCore_Tests_1b.xml", "exp_IsacCore_Tests_1c.xml" };
	
	/**
	 * Index of the object which will be the next to be shown to the user.
	 * If session is not active, than index is -1. 
	 */
	private int objectindex_ = -1;

	/**
	 * Keeps inforamtion which type of session is currently running:
	 * "assessment" or "course", or null if no session is active.
	 * Maybe enum would be better than string. 
	 */
	private String sessiontype_ = null;
	
	/**
	 * Keeps information about the session state.
	 */
	private SessionState sessionstate_ = null;

	/**
	 * Keeps availability information about knowledge and competence state. 
	 */
	private KnowledgeStateQuality knowledgestatequality_ = null;
	
	
	// --- constructor ---
	
	/**
	 * Constructor: initialisation
	 */
	public CompodDummy ()
	{
		// initialisation
		
		// no session at the beginning
		sessionstate_ = SessionState.INACTIVE;

		// empty set at beginning, empty set is also a possible knowledge state
		knowledgestatequality_ = KnowledgeStateQuality.AVAILABLE; 
	}
	
	/**
	 * @see ICompod#setLMSID(java.lang.String)
	 */
	public void setLMSID(String lmsid) throws Exception
	{
		lmsid_ = lmsid;
	}

    /**
     * @see isac.session.ICompod#setScopeByTopic(java.lang.String, java.lang.String)
     */
    public void setScopeByTopic (String username, String topic) throws Exception
    {
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
    	// not used in dummy
    }

	/**
	 * @see ICompod#startSession(java.lang.String, java.lang.String)
	 */
	public boolean startSession(String username, String sessiontype) throws Exception
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		// user is ignored in this dummy
		
		// if a session is already running then return 
		if (sessionstate_ == SessionState.ACTIVE)
			return false;
	
		// activate and initialise session
		sessionstate_ = SessionState.ACTIVE;
		sessiontype_ = sessiontype;
		objectindex_ = 0; 
		// during an assessment, the knowledge and competence state is not available
		if (sessiontype_ == "assessment")
			knowledgestatequality_ = KnowledgeStateQuality.UNAVAILABLE;
		
		return true;
	}

	/**
	 * @see ICompod#stopSession(java.lang.String, boolean)
	 */
	public boolean stopSession(String username, boolean cancel) throws Exception
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");

		// user and cancel type is ignored in this dummy
		
		// if a session is not running then return 
		if (sessionstate_ == SessionState.INACTIVE)
			return false;
	
		// stop session
		sessionstate_ = SessionState.INACTIVE;
		knowledgestatequality_ = KnowledgeStateQuality.PRELIMINARY;
		objectindex_ = -1; 
		
		return true;
	}
	

	/**
	 * @see ICompod#getSessionStatus(java.lang.String)
	 */
	public SessionState getSessionState(String username) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		return sessionstate_;
	}


	/**
	 * @see ICompod#getNextObjects(java.lang.String)
	 */
	public String[] getNextObjects(String username) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		// return if no session is active
		if (sessionstate_ == SessionState.INACTIVE)
			return null;
		
		// system invariant: this case should not happen
		int objectnumber = objects_.length;
		if (objectindex_ < 0 || objectindex_ >= objectnumber)
			return null; 
		
		// return next objects
		String[] nextobjects = new String[1];
		nextobjects[0] = objects_[objectindex_];
		
		// different behaviour if a course is conducted
		// in contrast to the assessment, there is no separate update,
		// therefore the update has to be done here.
		if (sessiontype_ == "course")
		{
			// update and prepare for getNextObjects
			objectindex_++;
			
			// stop if no more objects are available
			if (objectindex_ >= objectnumber)
			{
				sessionstate_ = SessionState.INACTIVE;
				objectindex_ = -1; 
			}
		}
		return nextobjects;
	}
	
	/**
	 * @see ICompod#notifyObjectResult(java.lang.String, java.lang.String, boolean)
	 */
	public void notifyObjectResult(String username, String objectid,
			boolean solved) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		// TODO write data in a file (or on the console)
		
		// update and prepare for getNextObjects
		objectindex_++;
		
		// stop if no more objects are available
		int objectnumber = objects_.length;
		if (objectindex_ >= objectnumber)
		{
			sessionstate_ = SessionState.INACTIVE;
			knowledgestatequality_ = KnowledgeStateQuality.AVAILABLE;
			objectindex_ = -1; 
		}
	}

	/**
	 * @see ICompod#notifyObjectMicroResult(java.lang.String, java.lang.String, java.lang.String, java.lang.String, boolean)
	 */
	public void notifyObjectMicroResult(String username, String objectid,
			String skillid, String masterylevel, boolean solved) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		// TODO write data in a file (or on the console)
	}
	
	
	/**
	 * @see ICompod#getKnowledgeStateQuality(java.lang.String)
	 */
	public KnowledgeStateQuality getKnowledgeStateQuality(String username) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		return knowledgestatequality_;
	}

	/**
	 * @see ICompod#getKnowledgeState(java.lang.String)
	 */
	public String[] getKnowledgeState(String username) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		String[] knowledgestate = null;
		if (knowledgestatequality_ != KnowledgeStateQuality.UNAVAILABLE)
			knowledgestate = new String[0];
		
		return knowledgestate;
	}

	/**
	 * @see ICompod#getCompetenceState(java.lang.String)
	 */
	public String[] getCompetenceState(String username) throws Exception 
	{
		if (lmsid_ == null)
			throw new Exception ("lmsid is not set");
		
		String[] competencestate = null;
		if (knowledgestatequality_ != KnowledgeStateQuality.UNAVAILABLE)
			competencestate = new String[0];
		
		return competencestate;
	}


	
	/**
	 * This main method is used for 
	 * 1) testing the dummy implementation and 
	 * 2) demonstrating the usage of the ICompod interface.
	 * This could also be placed in an external class.
	 * 
	 * @param args 
	 * 		(Command line arguments) not used
	 */
	public static void main (String[] args)
	{
		CompodDummy compod = new CompodDummy ();
		
		System.out.print("Compod Dummy.\n");
		
		// initialisation
		String lmsid = "isac-beta-xxx";
		String username = "some wired user";
		
		try
		{
			compod.setLMSID (lmsid);
		} 
		catch (Exception e)
		{
			e.printStackTrace();
			return; // stop
		}

		// assessment
		try
		{
			System.out.print("- Assessment.\n");
			compod.startSession (username, "assessment");
			while (compod.getSessionState (username) == SessionState.ACTIVE)
			{
				String[] nextobjects = compod.getNextObjects (username);
				
				// output
				for (int i = 0; i < nextobjects.length; i++)
					System.out.print (nextobjects[i]);
				System.out.println ("");
				
				compod.notifyObjectResult(username, nextobjects[0], true);
			}
		} 
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		// course
		try
		{
			System.out.print("- Course.\n");
			compod.startSession (username, "course");
			while (compod.getSessionState (username) == SessionState.ACTIVE)
			{
				String[] nextobjects = compod.getNextObjects (username);
				
				// output
				for (int i = 0; i < nextobjects.length; i++)
					System.out.print (nextobjects[i]);
				System.out.println ("");
			}
		} 
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		
	}




	
	
	
	
	
	
}
